import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.Choice;
import jv.object.PsDebug;
import jv.object.PsUpdateIf;
import jv.project.PjProject_IP;
import jv.vecmath.PdVector;

/**
 * Info panel of project. Each project, as well as each geometry, may have an
 * info panel for inspecting and steering the instance. Modifications in the
 * panel should result in a call to <code>project.update(Object)</code>. This
 * info panel is optional and can be removed without any changes in the project.
 * Each info panel must follow special naming conventions such that JavaView is
 * able to show it automatically, or on request. The class name must be
 * identical to the class name of the corresponding class with the suffix '_IP'
 * attached. IP means InfoPanel.
 * 
 * @see jv.project.PjProject_IP
 * @author Simon Gamper
 * @version 16.02.2011, 1.0
 * 
 */
public class BccProject_IP extends PjProject_IP implements ActionListener,
		ItemListener {
	/**
	 * Extend basic class PjProject_IP
	 */
	private static final long serialVersionUID = 11L;
	protected BccProject bccProject;
	protected Panel sliderPanel;
	protected Button bReset;
	protected Choice m_cSwitch;
	protected Checkbox m_ShowPrimitive;
	protected Checkbox m_ShowLattice;
	protected Checkbox m_ShowKVectors;
	protected Checkbox m_ShowSymmetry;
	protected Checkbox m_ShowEuclidian;
	protected Panel pDistance;
	/** Display distances*/
	protected Label Distance_gamma_1;
	protected Label Distance_gamma_2;
	protected Label Distance_gamma_3;
	protected Label Distance_gamma_4;
	protected Label Distance_gamma_5;
	protected Label Distance_gamma_1_label;
	protected Label Distance_gamma_2_label;
	protected Label Distance_gamma_3_label;
	protected Label Distance_gamma_4_label;
	protected Label Distance_gamma_5_label;
	protected Button m_bRun;
	protected Button m_help;
	protected Button m_about;
	/**
	 * Save background color of button m_bRun as long as minimization is
	 * running.
	 */
	private final Color m_bRunColor;

	public BccProject_IP() {
		/**constructor*/
		super();
		addTitle("");
		// Panel to contain sliders
		sliderPanel = new Panel(new GridLayout(1, 1));
		add(sliderPanel);
		
		// just add text
		final Panel pShow_title = new Panel();
		pShow_title.add(new Label("Set Visible"));
		add(pShow_title);
		
		// add checkboxes to activate geometries
		final Panel pShow = new Panel(new GridLayout(5, 1));
		add(pShow);
		{
			m_ShowPrimitive = new Checkbox("Primitive cell");
			m_ShowPrimitive.addItemListener(this);
			pShow.add(m_ShowPrimitive);
			m_ShowLattice = new Checkbox("Lattice");
			m_ShowLattice.addItemListener(this);
			pShow.add(m_ShowLattice);
			m_ShowKVectors = new Checkbox("Translation vectors");
			m_ShowKVectors.addItemListener(this);
			pShow.add(m_ShowKVectors);
			m_ShowSymmetry = new Checkbox("Symmetry elements");
			m_ShowSymmetry.addItemListener(this);
			pShow.add(m_ShowSymmetry);
			m_ShowEuclidian = new Checkbox("First brioullin zone");
			m_ShowEuclidian.addItemListener(this);
			pShow.add(m_ShowEuclidian);

		}
		
		// just add text
		Panel choice = new Panel();
		choice.add(new Label("Select space: "));
		add(choice);
		
		// add choice to switch between real and reciprocal space
		final Panel pSwitches = new Panel();
		m_cSwitch = new Choice();
		m_cSwitch.addItemListener(this);
		pSwitches.add(m_cSwitch);
		add(pSwitches);
		
		// just add text
		Panel distance = new Panel();
		distance.add(new Label("Distance of importand symmetry points: "));
		add(distance);
		
		// panel with distances of important symmetry points
		pDistance = new Panel(new GridLayout(6, 2));
		add(pDistance);
		{
			Distance_gamma_1_label = new Label("|\u0393N|:");// textfield
			pDistance.add(Distance_gamma_1_label);// value
			Distance_gamma_1 = new Label();
			pDistance.add(Distance_gamma_1);
			Distance_gamma_2_label = new Label("|\u0393H|:");
			pDistance.add(Distance_gamma_2_label);
			Distance_gamma_2 = new Label();
			pDistance.add(Distance_gamma_2);
			Distance_gamma_3_label = new Label("|\u0393P|:");
			pDistance.add(Distance_gamma_3_label);
			Distance_gamma_3 = new Label();
			pDistance.add(Distance_gamma_3);
			Distance_gamma_4_label = new Label();
			pDistance.add(Distance_gamma_4_label);
			Distance_gamma_4 = new Label();
			pDistance.add(Distance_gamma_4);
			Distance_gamma_5_label = new Label();
			pDistance.add(Distance_gamma_5_label);
			Distance_gamma_5 = new Label();
			pDistance.add(Distance_gamma_5);

		}
		
		// buttons at bottom for animation and resetting and shortcut view
		final Panel border = new Panel(new FlowLayout());
		add(border);
		final Panel pBottomButtons = new Panel(
				new GridLayout(3,1));
		{border.add(pBottomButtons);
		m_bRun = new Button("Stop");
		m_bRun.addActionListener(this);
		m_bRunColor = m_bRun.getBackground();
		m_bRun.setBackground(Color.red);
		pBottomButtons.add(m_bRun);

		bReset = new Button("Reset");
		bReset.addActionListener(this);
		pBottomButtons.add(bReset);

		m_help = new Button("Shortcuts");
		m_help.addActionListener(this);
		pBottomButtons.add(m_help);}
		
		final Panel pabout = new Panel(new FlowLayout(FlowLayout.CENTER));
		add(pabout);
		m_about = new Button("about applet");
		m_about.addActionListener(this);
		m_about.setFont(new Font("serif", Font.BOLD, 12));
		pabout.add(m_about);
		if (getClass() == BccProject_IP.class)
			init();
	}

	public void init() {
		super.init();
	}

	// set active parents
	public void setParent(final PsUpdateIf parent) {
		super.setParent(parent);
		bccProject = (BccProject) parent;
		// choice init.
		m_cSwitch.removeAll();
		String[] listtypes = bccProject.m_types;
		if (listtypes != null) {
			int len = listtypes.length;
			for (int i = 0; i < len; i++)
				m_cSwitch.add(listtypes[i]);
		}
		// set current title of IP
		setTitle(bccProject.getName());
		sliderPanel.add(bccProject.m_a.getInfoPanel());
	}

	/**
	 * Here we arrive from outside world of this panel, e.g. if project has
	 * changed somewhere else and must update its panel. Such an update is
	 * automatically by superclasses of PjProject.
	 */
	public boolean update(final Object event) {
		if (bccProject == null) {
			if (PsDebug.WARNING)
				PsDebug.warning("missing parent, setParent not called");
			return false;
		}
		if (event == bccProject) {
			// get (name of) project and set choice
			m_cSwitch.select(bccProject.gettypeName());
			// set current title
			setTitle("Viewing " + bccProject.gettypeName());
			m_ShowPrimitive.setState(bccProject.m_ShowPrimitive_a);
			m_ShowLattice.setState(bccProject.m_ShowLattice_a);
			m_ShowKVectors.setState(bccProject.m_ShowKVectors_a);
			m_ShowSymmetry.setState(bccProject.m_ShowSymmetry_a);
			m_ShowEuclidian.setState(bccProject.m_ShowEuclidian_a);

			if (m_cSwitch.getSelectedItem().startsWith("Real")) {
				// update value fields of importand symmetry points real space
				// choice
				Distance_gamma_1.setText(String
						.valueOf(bccProject.m_real.m_Distance_gamma_1));
				Distance_gamma_3.setText(String
						.valueOf(bccProject.m_real.m_Distance_gamma_3));
				Distance_gamma_2.setText(String
						.valueOf(bccProject.m_real.m_Distance_gamma_2));
				Distance_gamma_4.setText(String
						.valueOf(bccProject.m_real.m_Distance_gamma_4));
				Distance_gamma_5.setText(String
						.valueOf(bccProject.m_real.m_Distance_gamma_5));
			}
			// update value fields of importand symmetry points real space
			// choice
			else if (m_cSwitch.getSelectedItem().startsWith("Reciprocal")) {
				Distance_gamma_1.setText(String
						.valueOf(bccProject.m_reciprocal.m_Distance_gamma_1));
				Distance_gamma_3.setText(String
						.valueOf(bccProject.m_reciprocal.m_Distance_gamma_3));
				Distance_gamma_2.setText(String
						.valueOf(bccProject.m_reciprocal.m_Distance_gamma_2));
			}

			return true;
		}
		return super.update(event);
	}

	/**
	 * Handle action events invoked from buttons, menu items, text fields.
	 */
	public void actionPerformed(final ActionEvent event) {
		if (bccProject == null)
			return;
		final Object source = event.getSource();
		if (source == bReset) {
			bccProject.init();
			//set camera default view
			bccProject.getDisplay().getCamera().setDist(11.5);
			bccProject.getDisplay().getCamera().setRoll(0.);
			bccProject.getDisplay().getCamera().setViewDir(new PdVector(-.5,-.5,-.5));
	
			bccProject.update(this);
			

		} else if (source == m_bRun) { // run automation button
			if (m_bRun.getLabel().equals("Run")) {
				m_bRun.setBackground(Color.red);
				m_bRun.setLabel("Stop");
				bccProject.start();
			} else {
				if (m_bRun.getLabel().equals("Stop"))
					m_bRun.setBackground(m_bRunColor);
				m_bRun.setLabel("Run");
				bccProject.stopAnim();
			}
		}
		else if (source == m_help) { // show shortcuts panel
			bccProject.getViewer().showDialog(36);	
			
		}
		else if (source == m_about) { // show shortcuts panel
			bccProject.getViewer().showDialog(32);	
			
		}
	}

	/**
	 * Handle item events invoked from choice, checkboxes...
	 */
	public void itemStateChanged(final ItemEvent event) {
		if (bccProject == null)
			return;
		final Object source = event.getSource();
		if (source == m_cSwitch) {
			//update title of IP if switch
			bccProject.settypeName(m_cSwitch.getSelectedItem());
			setTitle("Viewing " + bccProject.gettypeName());
			//update geometry of IP if switch
			bccProject.updateGeometry(bccProject.gettypeName());
			//update rest...
			if (m_cSwitch.getSelectedItem().startsWith("Reciprocal")) {
				m_ShowEuclidian.setLabel("First brioulin zone");
				Distance_gamma_1_label.setText("|\u0393N|");
				Distance_gamma_2_label.setText("|\u0393H|");
				Distance_gamma_3_label.setText("|\u0393P|");
				Distance_gamma_4_label.setText("");
				Distance_gamma_4.setText("");
				Distance_gamma_5_label.setText("");
				Distance_gamma_5.setText("");
				Distance_gamma_1.setText(String
						.valueOf(bccProject.m_real.m_Distance_gamma_1));
				Distance_gamma_2.setText(String
						.valueOf(bccProject.m_real.m_Distance_gamma_2));
				Distance_gamma_3.setText(String
						.valueOf(bccProject.m_real.m_Distance_gamma_3));
			} else {
				m_ShowEuclidian.setLabel("Wigner Seitz cell");
				Distance_gamma_4_label.setText("|\u0393W|");
				Distance_gamma_1_label.setText("|\u0393L|");
				Distance_gamma_2_label.setText("|\u0393X|");
				Distance_gamma_3_label.setText("|\u0393K|");
				Distance_gamma_5_label.setText("|\u0393U|");
				Distance_gamma_1.setText(String
						.valueOf(bccProject.m_real.m_Distance_gamma_1));
				Distance_gamma_2.setText(String
						.valueOf(bccProject.m_real.m_Distance_gamma_2));
				Distance_gamma_3.setText(String
						.valueOf(bccProject.m_real.m_Distance_gamma_3));
				Distance_gamma_4.setText(String
						.valueOf(bccProject.m_real.m_Distance_gamma_4));
				Distance_gamma_5.setText(String
						.valueOf(bccProject.m_real.m_Distance_gamma_5));

			}
		}

		if (source == m_ShowPrimitive || source == m_cSwitch) {
			if (PsDebug.NOTIFY)
				PsDebug.notify("switch show primitive cell");
			bccProject.m_ShowPrimitive_a = m_ShowPrimitive.getState();
			if (m_cSwitch.getSelectedItem().startsWith("Real")) {
				if (bccProject.m_ShowPrimitive_a) {
					bccProject.addGeometry(bccProject.m_real.m_edge);
				} else {
					bccProject.removeGeometry(bccProject.m_real.m_edge);
				}
			}
			if (m_cSwitch.getSelectedItem().startsWith("Reciprocal")) {
				if (bccProject.m_ShowPrimitive_a) {
					bccProject.addGeometry(bccProject.m_reciprocal.m_edge);
				} else {
					bccProject.removeGeometry(bccProject.m_reciprocal.m_edge);
				}

			}
		}

		if (source == m_ShowLattice || source == m_cSwitch) {
			if (PsDebug.NOTIFY)
				PsDebug.notify("switch show Lattice");
			bccProject.m_ShowLattice_a = m_ShowLattice.getState();
			if (m_cSwitch.getSelectedItem().startsWith("Real")) {
				if (bccProject.m_ShowLattice_a) {
					bccProject.addGeometry(bccProject.m_real.m_geom);
				} else {
					bccProject.removeGeometry(bccProject.m_real.m_geom);
				}
			} else if (m_cSwitch.getSelectedItem().startsWith("Reciprocal")) {
				if (bccProject.m_ShowLattice_a) {
					bccProject.addGeometry(bccProject.m_reciprocal.m_geom);
				} else {
					bccProject.removeGeometry(bccProject.m_reciprocal.m_geom);
				}
			}

		}

		if (source == m_ShowKVectors || source == m_cSwitch) {
			bccProject.m_ShowKVectors_a = m_ShowKVectors.getState();

			if (m_cSwitch.getSelectedItem().startsWith("Real")) {
				if (bccProject.m_ShowKVectors_a) {
					bccProject.addGeometry(bccProject.m_real.m_kdir);
				} else {
					bccProject.removeGeometry(bccProject.m_real.m_kdir);
				}

			} else if (m_cSwitch.getSelectedItem().startsWith("Reciprocal")) {
				if (bccProject.m_ShowKVectors_a) {
					bccProject.addGeometry(bccProject.m_reciprocal.m_kdir);
				} else {
					bccProject.removeGeometry(bccProject.m_reciprocal.m_kdir);
				}

			}

		}
		if (source == m_ShowSymmetry || source == m_cSwitch) {
			bccProject.m_ShowSymmetry_a = m_ShowSymmetry.getState();

			if (m_cSwitch.getSelectedItem().startsWith("Real")) {
				if (bccProject.m_ShowSymmetry_a) {
					bccProject.addGeometry(bccProject.m_real.m_symmetry);
				} else {
					bccProject.removeGeometry(bccProject.m_real.m_symmetry);
				}

			} else if (m_cSwitch.getSelectedItem().startsWith("Reciprocal")) {
				if (bccProject.m_ShowSymmetry_a) {
					bccProject.addGeometry(bccProject.m_reciprocal.m_symmetry);
				} else {
					bccProject
							.removeGeometry(bccProject.m_reciprocal.m_symmetry);
				}

			}

		}
		if (source == m_ShowEuclidian || source == m_cSwitch) {
			bccProject.m_ShowEuclidian_a = m_ShowEuclidian.getState();

			if (m_cSwitch.getSelectedItem().startsWith("Real")) {
				if (bccProject.m_ShowEuclidian_a) {
					bccProject.addGeometry(bccProject.m_real.m_brioullin);
				} else {
					bccProject.removeGeometry(bccProject.m_real.m_brioullin);
				}

			} else if (m_cSwitch.getSelectedItem().startsWith("Reciprocal")) {
				if (bccProject.m_ShowEuclidian_a) {
					bccProject.addGeometry(bccProject.m_reciprocal.m_brioullin);
				} else {
					bccProject
							.removeGeometry(bccProject.m_reciprocal.m_brioullin);
				}

			}

		}
		return;
	}

}
