import java.awt.Color;
import jv.geom.PgPointSet;
import jv.geom.PgPolygonSet;
import jv.vecmath.PiVector;
import jv.vecmath.PdVector;

public class ReciprocalLattice extends PgPointSet {
	/**
	 * Class to generate geometries of reciprocal bcc lattice for bcc applet
	 * 
	 * @see PgPointSet
	 * @author Simon Gamper
	 * @version 14.02.2011, 1.0
	 */
	private static final long serialVersionUID = 1L;
	protected PgPointSet m_geom; // Poly of geometry
	protected PgPointSet m_element;
	protected PgPolygonSet m_edge;
	protected PgPolygonSet m_kdir;
	protected PgPolygonSet m_brioullin;
	protected PgPolygonSet m_symmetry;
	protected int m_numUPoints = 5;
	protected int m_numVPoints = 5;
	protected int m_numWPoints = 5;
	protected double m_Distance_gamma_1;
	protected double m_Distance_gamma_2;
	protected double m_Distance_gamma_3;

	public ReciprocalLattice() {

		m_geom = new PgPointSet(3);
		m_geom.setName("Lattice");

		m_element = new PgPointSet(3);
		m_element.setName("Brioullin");

		m_edge = new PgPolygonSet(3);
		m_edge.setName("Primitive Cell");

		m_kdir = new PgPolygonSet(3);
		m_kdir.setName("k-Diretion vectors");

		m_brioullin = new PgPolygonSet(3);
		m_brioullin.setName("brioullin zone");

		m_symmetry = new PgPolygonSet(3);
		m_symmetry.setName("symmetry points");
		init();
	}

	public void init(double a) {

		m_brioullin.setNumVertices(14);
		m_brioullin.setGlobalVertexSize(1.);
		m_brioullin.setGlobalVertexColor(Color.green);
		m_brioullin.setNumPolygons(1);
		m_brioullin.setVertex(0, a / 2, 0, 0);
		m_brioullin.setVertex(1, -a / 2, 0, 0);
		m_brioullin.setVertex(2, 0, a / 2, 0);
		m_brioullin.setVertex(3, 0, -a / 2, 0);
		m_brioullin.setVertex(4, 0, 0, a / 2);
		m_brioullin.setVertex(5, 0, 0, -a / 2);
		m_brioullin.setVertex(6, a / 4, a / 4, a / 4);
		m_brioullin.setVertex(7, -a / 4, a / 4, a / 4);
		m_brioullin.setVertex(8, a / 4, -a / 4, a / 4);
		m_brioullin.setVertex(9, a / 4, a / 4, -a / 4);
		m_brioullin.setVertex(10, -a / 4, -a / 4, a / 4);
		m_brioullin.setVertex(11, -a / 4, a / 4, -a / 4);
		m_brioullin.setVertex(12, a / 4, -a / 4, -a / 4);
		m_brioullin.setVertex(13, -a / 4, -a / 4, -a / 4);
		int xa[] = { 0, 6, 2, 9, 0, 6, 4, 8, 0, 8, 3, 12, 0, 9, 5, 12, 0, 6, 2,
				7, 4, 10, 3, 13, 5, 11, 2, 7, 4, 10, 3, 10, 1, 13, 1, 11, 1, 7 };
		PiVector le = new PiVector(xa);
		m_brioullin.setPolygon(0, le);
		m_brioullin.setGlobalPolygonColor(Color.green);

		m_kdir.setNumVertices(4);
		m_kdir.setNumPolygons(3);
		m_kdir.setVertex(0, 0, 0, 0);
		m_kdir.setPolygon(0, new PiVector(0, 1));
		m_kdir.getPolygon(0).setName("b1");
		m_kdir.setPolygon(1, new PiVector(0, 2));
		m_kdir.getPolygon(1).setName("b2");
		m_kdir.setPolygon(2, new PiVector(0, 3));
		m_kdir.getPolygon(2).setName("b3");
		m_kdir.setVertex(1, 0, a / 2, a / 2);
		m_kdir.setVertex(2, a / 2, 0, a / 2);
		m_kdir.setVertex(3, a / 2, a / 2, 0);
		m_kdir.setGlobalPolygonColor(Color.blue);
		m_kdir.showPolygonEndArrow(true);
		m_kdir.showPolygonLabels(true);

		// init symmetry points
		m_symmetry.setNumVertices(4);

		m_symmetry.setVertex(0, new PdVector(0, 0, 0));
		m_symmetry.getVertex(0).setName("\u0393");
		PdVector H = PdVector.copyNew(m_brioullin.getVertex(2));
		m_symmetry.setVertex(1, H);
		m_symmetry.getVertex(1).setName("H");
		PdVector P = PdVector.copyNew(m_brioullin.getVertex(6));
		m_symmetry.setVertex(2, P);
		m_symmetry.getVertex(2).setName("P");
		PdVector N = PdVector.copyNew(m_kdir.getVertex(3));
		N.multScalar(0.5);
		m_symmetry.setVertex(3, N);
		m_symmetry.getVertex(3).setName("N");

		m_symmetry.showVertexLabels(true);
		m_symmetry.showPolygonLabels(true);
		m_symmetry.setNumPolygons(6);
		m_symmetry.setPolygon(0, new PiVector(0, 1));
		m_symmetry.getPolygon(0).setName("\u039B");
		m_symmetry.setPolygon(1, new PiVector(0, 2));
		m_symmetry.getPolygon(1).setName("\u0394");
		m_symmetry.setPolygon(2, new PiVector(1, 2));
		m_symmetry.getPolygon(2).setName("F");
		m_symmetry.setPolygon(3, new PiVector(0, 3));
		m_symmetry.getPolygon(3).setName("\u03A3");
		m_symmetry.setPolygon(4, new PiVector(2, 3));
		m_symmetry.getPolygon(4).setName("D");
		m_symmetry.setPolygon(5, new PiVector(1, 3));
		m_symmetry.getPolygon(5).setName("G");

		m_geom.setGlobalVertexColor(Color.blue);
		m_geom.setNumVertices(m_numUPoints * m_numVPoints * m_numWPoints);
		m_geom.setVertexColor(
				(m_numUPoints * m_numVPoints * m_numWPoints + 1) / 2,
				Color.yellow);

		m_edge.setGlobalVertexColor(Color.yellow);
		m_edge.setGlobalVertexSize(4.);
		m_edge.setGlobalVertexSize(3.);
		computePoly(a); // Compute initial surface
		m_element.setGlobalVertexSize(1.);
		computebrioullin(a);

	}

	public void computePoly(double b) {
		// Compute lattice points
		double a = 2 * Math.PI / b;
		PdVector x = new PdVector(0, a / 2, a / 2);
		PdVector y = new PdVector(a / 2, 0, a / 2);
		PdVector z = new PdVector(a / 2, a / 2, 0);
		PdVector t = new PdVector(3);

		int ind = 0;
		int length = (int) Math.floor(m_numUPoints / 2);
		for (int i = -length; i <= length; i++) {
			for (int j = -length; j <= length; j++) {
				for (int k = -length; k <= length; k++) {
					t = PdVector.blendNew(i, x, j, y, k, z);
					m_geom.setVertex(ind, t);
					ind++;
				}
			}
		}
	}

	public void computebrioullin(double b) {
		// compute points of lattice
		double a = 2 * Math.PI / b;
		m_element.setNumVertices(29);
		m_edge.setNumPolygons(6);
		m_edge.setNumVertices(30);

		m_kdir.setVertex(1, 0, a / 2, a / 2);
		m_kdir.setVertex(2, a / 2, 0, a / 2);
		m_kdir.setVertex(3, a / 2, a / 2, 0);

		// compute points of brioullin zone
		m_brioullin.setVertex(0, a / 2, 0, 0);
		m_brioullin.setVertex(1, -a / 2, 0, 0);
		m_brioullin.setVertex(2, 0, a / 2, 0);
		m_brioullin.setVertex(3, 0, -a / 2, 0);
		m_brioullin.setVertex(4, 0, 0, a / 2);
		m_brioullin.setVertex(5, 0, 0, -a / 2);
		m_brioullin.setVertex(6, a / 4, a / 4, a / 4);
		m_brioullin.setVertex(7, -a / 4, a / 4, a / 4);
		m_brioullin.setVertex(8, a / 4, -a / 4, a / 4);
		m_brioullin.setVertex(9, a / 4, a / 4, -a / 4);
		m_brioullin.setVertex(10, -a / 4, -a / 4, a / 4);
		m_brioullin.setVertex(11, -a / 4, a / 4, -a / 4);
		m_brioullin.setVertex(12, a / 4, -a / 4, -a / 4);
		m_brioullin.setVertex(13, -a / 4, -a / 4, -a / 4);

		PdVector x1 = new PdVector(0, a / 4, a / 4);
		PdVector y1 = new PdVector(a / 4, 0, a / 4);
		PdVector z1 = new PdVector(a / 4, a / 4, 0);
		PdVector t = new PdVector(3);

		int ind = 0;
		for (int i = -1; i <= 1; i++) {
			for (int j = -1; j <= 1; j++) {
				for (int k = -1; k <= 1; k++) {
					t = PdVector.blendNew(i, x1, j, y1, k, z1);
					if (t.getEntry(0) == 0. | t.getEntry(1) == 0.
							| t.getEntry(2) == 0.) {
						m_element.setVertex(ind, t);
						ind++;
					}
				}
			}
		}
		m_element.setVertex(28, 3 / 8 * a, 3 / 8 * a, 3 / 8 * a);
		m_element.setVertexColor(28, Color.yellow);

		ind = 0;
		int x = 1;
		int y = 1;
		int z = 1;
		for (int i = 0; i < 6; i++) {
			PiVector line = m_edge.getPolygon(i);
			line.setSize(5);
			if (i % 2 == 0)
				x = -1;
			else
				x = 1;
			for (int j = 0; j <= 4; j++) {
				switch (j) {
				case 0: {
					y = -1;
					z = -1;
					break;
				}
				case 1: {
					y = -1;
					z = 1;
					break;
				}
				case 2: {
					y = 1;
					z = 1;
					break;
				}
				case 3: {
					y = 1;
					z = -1;
					break;
				}
				case 4: {
					y = -1;
					z = -1;
					break;
				}
				}
				if (i == 1 | i == 2) {
					t = PdVector.blendNew(x, x1, y, y1, z, z1);
				} else if (i == 3 | i == 4) {
					t = PdVector.blendNew(z, x1, x, y1, y, z1);
				} else if (i == 5 | i == 6) {
					t = PdVector.blendNew(y, x1, z, y1, x, z1);
				}
				line.m_data[j] = ind;
				m_edge.setVertex(ind, t);
				ind++;
			}
		}

		m_symmetry.getVertex(0).setName("\u0393");
		PdVector H = PdVector.copyNew(m_brioullin.getVertex(2));
		m_symmetry.setVertex(1, H);
		m_symmetry.getVertex(1).setName("H");
		PdVector P = PdVector.copyNew(m_brioullin.getVertex(6));
		m_symmetry.setVertex(2, P);
		m_symmetry.getVertex(2).setName("P");
		PdVector N = PdVector.copyNew(m_kdir.getVertex(3));
		N.multScalar(0.5);
		m_symmetry.setVertex(3, N);
		m_symmetry.getVertex(3).setName("N");
		m_Distance_gamma_1 = PdVector.dist(m_symmetry.getVertex(0),
				m_symmetry.getVertex(1));
		m_Distance_gamma_2 = PdVector.dist(m_symmetry.getVertex(0),
				m_symmetry.getVertex(2));
		m_Distance_gamma_3 = PdVector.dist(m_symmetry.getVertex(0),
				m_symmetry.getVertex(3));

	}

}
